@@ -4,23 +4,25 @@
 //
 // SPDX-License-Identifier:    MIT
 
-#include <vector>
-#include <unistd.h>
+#include <chrono>
+#include <dolfinx/common/log.h>
+#include <fstream>
 #include <ios>
 #include <iostream>
-#include <thread>
-#include <chrono>
-#include <fstream>
-#include <string>
 #include <iterator>
-#include <dolfinx/common/log.h>
+#include <string>
+#include <thread>
+#include <unistd.h>
+#include <vector>
 
 void process_mem_usage(bool& quit)
 {
-  loguru::set_thread_name("MEMORY");
+  std::string fmt = "[%Y-%m-%d %H:%M:%S.%e] [MEM] [%l] %v";
+  spdlog::set_pattern(fmt);
+
   const int page_size_bytes = sysconf(_SC_PAGE_SIZE);
 
-  while(!quit)
+  while (!quit)
   {
     std::ifstream f("/proc/self/stat", std::ios_base::in);
     std::istream_iterator<std::string> it(f);
@@ -29,9 +31,8 @@ void process_mem_usage(bool& quit)
     std::size_t vsize, rss;
     f >> vsize >> rss;
     f.close();
-    LOG(WARNING) << "VSIZE=" << vsize/1024 << " RSS=" << rss*page_size_bytes/1024 ;
+    spdlog::warn("VSIZE={}, RSS={}", vsize / 1024,
+                 rss * page_size_bytes / 1024);
     std::this_thread::sleep_for(std::chrono::milliseconds(100));
   }
 }
-
-
